#define	yymaxdepth sshmaxdepth
#define	yyparse	sshparse
#define	yylex	sshlex
#define	yyerror	ssherror
#define	yylval	sshlval
#define	yychar	sshchar
#define	yydebug	sshdebug
#define	yypact	sshpact
#define	yyr1	sshr1
#define	yyr2	sshr2
#define	yydef	sshdef
#define	yychk	sshchk
#define	yypgo	sshpgo
#define	yyact	sshact
#define	yyexca	sshexca
#define yyerrflag ssherrflag
#define yynerrs	sshnerrs
#define	yyps	sshps
#define	yypv	sshpv
#define	yys	sshs
#define	yy_yys	sshyys
#define	yystate	sshstate
#define	yytmp	sshtmp
#define	yyv	sshv
#define	yy_yyv	sshyyv
#define	yyval	sshval
#define	yylloc	sshlloc
#define yyreds	sshreds
#define yytoks	sshtoks
#define yylhs	sshyylhs
#define yylen	sshyylen
#define yydefred sshyydefred
#define yydgoto	sshyydgoto
#define yysindex sshyysindex
#define yyrindex sshyyrindex
#define yygindex sshyygindex
#define yytable	 sshyytable
#define yycheck	 sshyycheck
#define yyname   sshyyname
#define yyrule   sshyyrule
